import java.util.*;
import java.io.*;

class Image{
	Pixel [][] pixels;
	int width, height,max;
	public Image(int w, int h, int m){ width =w; height =h; max=m; }
	public Image(String filename){ //Reads image from PPM file
		File file = new File(filename);
		Scanner sc = null;
		try{
			sc = new Scanner(file);
			String start = sc.next();
			if(!"P3".equals(start)){ System.out.println("Bad file type!"); return; }
			width = sc.nextInt();
			height = sc.nextInt();
			max = sc.nextInt();
			pixels = new Pixel [height] [width];
			for(int i=0; i<height; i++)
				for(int j=0;j<width; j++){
				int r,g,b;
				r = sc.nextInt();
				g = sc.nextInt();
				b = sc.nextInt();
				pixels[i][j] = new Pixel(r,g,b,max);
			}
				
		}catch(IOException e){
			System.out.println("File I/O error!");
			e.printStackTrace();
		}finally{
			if(sc!=null) sc.close();
		}
	}

	public Image darken(){
		Image im = new Image(width,height,max);
		im.pixels = new Pixel [height] [width];
		for(int i=0; i<height; i++)
			for(int j=0; j<width; j++)
				im.pixels[i][j] = new Pixel(pixels[i][j].red/2, pixels[i][j].green/2,  pixels[i][j].blue/2, max);
		return im;
	}

	public Image lighten(){
		Image im = new Image(width,height,max);
		im.pixels = new Pixel [height] [width];
		for(int i=0; i<height; i++)
			for(int j=0; j<width; j++)
				im.pixels[i][j] = new Pixel(pixels[i][j].red*2, pixels[i][j].green*2,  pixels[i][j].blue*2, max);
		return im;
	}

	public Image blur(){
		Image im = new Image(width,height,max);
		im.pixels = new Pixel [height] [width];
		for(int j=0; j<width; j++) im.pixels[0][j] = new Pixel(pixels[0][j]);
		for(int j=0; j<width; j++) im.pixels[height-1][j] = new Pixel(pixels[height-1][j]);
		for(int i=0; i<height; i++) im.pixels[i][0] = new Pixel(pixels[i][0]);
		for(int i=0; i<height; i++) im.pixels[i][width-1] = new Pixel(pixels[i][width-1]);

		for(int i=1; i<height-1; i++)
			for(int j=1; j<width-1; j++){
				Pixel p = new Pixel(0,0,0,max);
				for(int k=i-1;k<=i+1;k++)
					for(int l=j-1;l<=j+1;l++){
						p.red += pixels[k][l].red;
						p.green += pixels[k][l].green;
						p.blue += pixels[k][l].blue;
					}
				p.red /= 9;
				p.green /= 9;
				p.blue /= 9;
				im.pixels[i][j] = p;	
		}
		return im;
	}

	public Image reverseX(){
		Image im = new Image(width,height,max);
		im.pixels = new Pixel [height] [width];
		for(int i=0; i<height; i++)
			for(int j=0; j<width; j++)
				im.pixels[i][j] = new Pixel(pixels[i][width-j-1]);
		return im;
	}

	public Image reverseY(){
		Image im = new Image(width,height,max);
		im.pixels = new Pixel [height] [width];
		for(int i=0; i<height; i++)
			for(int j=0; j<width; j++)
				im.pixels[i][j] = new Pixel(pixels[height-i-1][j]);
		return im;
	}
	public void write(String filename){
		PrintWriter writer = null;
		try{
			writer = new PrintWriter(filename);
			writer.println("P3");
			writer.println(width + " " + height);
			writer.println(max);
			for(int i=0;i<height;i++)
				for(int j=0;j<width;j++)
					writer.println(pixels[i][j]);
		}catch(IOException e){
			System.out.println("File I/O error (write)!");
			e.printStackTrace();
		}finally{
			if(writer!=null) writer.close();
		}
	}
	public static void main(String [] args){
		Image i = new Image(args[1]);
		if("lighten".equals(args[0]))
			i.lighten().write(args[2]);
		else if("darken".equals(args[0]))
			i.darken().write(args[2]);
		else if("blur".equals(args[0]))
			i.blur().blur().blur().write(args[2]);
		else if("reverseX".equals(args[0]))
			i.reverseX().write(args[2]);
		else if("reverseY".equals(args[0]))
			i.reverseY().write(args[2]);
	}
}

class Pixel{
	int red;
	int green;
	int blue;
	int max;
	public Pixel(int r, int g, int b, int m){
		red = r>m?m:r; green = g>m?m:g; blue = b>m?m:b; max = m;
	}
	public Pixel(Pixel p) { this(p.red,p.green,p.blue,p.max); }
	public Pixel() { this(0,0,0,255); }
	public Pixel(int r, int g, int b) { this(r,g,b,255); }
	public String toString() { String s=""; s+= red+"\n"+green+"\n"+blue; return s; }
}
